﻿--[[ $Id: PlayerMenu.lua 43377 2007-07-10 17:11:44Z hshh $ ]]--
-- ACE2 init
local L = AceLibrary("AceLocale-2.2"):new("PlayerMenu")
local bs = AceLibrary("Babble-Spell-2.2")
PlayerMenu = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceEvent-2.0", "AceHook-2.1", "AceConsole-2.0")
local Version = "3.1"
local _G = getfenv(0)

local SpellList = {}
SpellList["PRIEST"]={
	["Power Word: Fortitude"] = 1,
	["Prayer of Fortitude"] = 1,
	["Power Word: Shield"] = 1,
	["Divine Spirit"] = 1,
	["Prayer of Spirit"] = 1,
	["Shadow Protection"] = 1,
	["Prayer of Shadow Protection"] = 1,
	["Power Infusion"] = 1,
	["Cure Disease"] = 1,
	["Abolish Disease"] = 1,
	["Dispel Magic"] =1,
	["Resurrection"] = 1,
}
SpellList["DRUID"]={
	["Gift of the Wild"] = 1,
	["Mark of the Wild"] = 1,
	["Thorns"] = 1,
	["Remove Curse"] = 1,
	["Abolish Poison"] = 1,
	["Innervate"] = 1,
	["Rebirth"] = 1,
}
SpellList["PALADIN"]={
	["Redemption"] = 1,
	["Blessing of Freedom"] = 1,
	["Blessing of Kings"] = 1,
	["Blessing of Light"] = 1,
	["Blessing of Might"] = 1,
	["Blessing of Protection"] = 1,
	["Blessing of Sacrifice"] = 1,
	["Blessing of Salvation"] = 1,
	["Blessing of Sanctuary"] = 1,
	["Blessing of Wisdom"] = 1,
	["Greater Blessing of Kings"] = 1,
	["Greater Blessing of Light"] = 1,
	["Greater Blessing of Might"] = 1,
	["Greater Blessing of Salvation"] = 1,
	["Greater Blessing of Sanctuary"] = 1,
	["Greater Blessing of Wisdom"] = 1,
	["Divine Intervention"] = 1,
	["Cleanse"] = 1,
}
SpellList["WARLOCK"]={
	["Unending Breath"] = 1,
}
SpellList["MAGE"]={
	["Arcane Intellect"]=1,
	["Arcane Brilliance"] = 1,
	["Dampen Magic"] = 1,
	["Amplify Magic"] = 1,
	["Remove Lesser Curse"] = 1,
}


function PlayerMenu:OnInitialize()
	-- prepare Order_Orig, Button_Orig
	self.Order_Orig = { "WHISPER", "INVITE", "TARGET", "IGNORE", "REPORT_SPAM", "GUILD_PROMOTE", "GUILD_LEAVE", "GUILD_INVITE", "ADD_FRIEND", "GET_NAME", "WHO", "EXTMENU", "CANCEL" }
	self.Button_Orig = {}
	for k,v in pairs(self.Order_Orig) do
		self.Button_Orig[v]=1
	end
	self.Menus=table.getn(self.Order_Orig)

	-- save blizzard orign variables
	self.UnitPopupMenus={FRIEND={}, PLAYER={}, PARTY={}, RAID_PLAYER={}}
	for k,v in pairs(_G.UnitPopupMenus["FRIEND"]) do
		self.UnitPopupMenus["FRIEND"][k]=v
	end
	for k,v in pairs(_G.UnitPopupMenus["PLAYER"]) do
		self.UnitPopupMenus["PLAYER"][k]=v
	end
	for k,v in pairs(_G.UnitPopupMenus["PARTY"]) do
		self.UnitPopupMenus["PARTY"][k]=v
	end
	for k,v in pairs(_G.UnitPopupMenus["RAID_PLAYER"]) do
		self.UnitPopupMenus["RAID_PLAYER"][k]=v
	end

	-- register saved variables
	self:RegisterDB("PlayerMenu_Settings")
	-- init default variables
	self:RegisterDefaults("profile", {
		leftButton = false,
		order = self.Order_Orig,
		button = self.Button_Orig,
		extAutoHide = 3,
		useFocus = false,
	})
	-- found old saved variables, reset it.
	if (self.db.profile.version ~= Version) then
		self:ResetDB("profile")
		self:Print(L["MSG_RESETED"])
		self.db.profile.version = Version
	end

	-- register console slash commands
	self:RegisterChatCommand(
		{ "/playermenu", "/pm", "/apm" },
		{
			type = "group",
			args = {
				left_button = {
					order = 100,
					type = "toggle",
					name = L["Left Button Menu"],
					desc = L["leftButton_desc"],
					get = function() return self.db.profile.leftButton end,
					set = function(t)
						self.db.profile.leftButton = t
						if (self.db.profile.leftButton) then
							self:Print(L["MSG_LEFT_ON"])
						else
							self:Print(L["MSG_LEFT_OFF"])
						end
						self:Reload()
					end,
				},
				usefocus = {
					order = 200,
					type = "toggle",
					name = L["Use Focus to Cast"],
					desc = L["Use focus feature to cast spell instead of using target."],
					get = function() return self.db.profile.useFocus end,
					set = function(t) self.db.profile.useFocus = t end,
				},
				hide_delay = {
					order = 300,
					type = "range",
					name = L["Extend Menu Hide Delay"],
					min = 1,
					max = 20,
					step = 1,
					desc = L["Set extend menu auto hide delay time."],
					get = function() return self.db.profile.extAutoHide end,
					set = function(r)
						self.db.profile.extAutoHide=r
					end
				},
				menu_order = {
					order = 400,
					type = "text",
					name = L["Menu Order"],
					desc = L["order_desc"],
					usage = L["order_usage"],
					get = function()
						local order=""
						for k,v in pairs(self.db.profile.order) do
							if (self.db.profile.button[v]==1) then
								order=order..v..":1,"
							else
								order=order..v..":0,"
							end
						end
						order = string.sub(order, 1, -2)
						return order
					end,
					set = function (c)
						c = string.upper(c)
						local newOrder = {}
						local newButton = {}
						local i = 1
						for b,e in pairs(string.gmatch(c, "([%w_]+):(%d)")) do
							e = tonumber(e)
							if (e~=1 and e~=0) then
								e=1
							end
							if (b~=nil and self.Button_Orig[b]~=nil) then
								newOrder[i]=b
								newButton[b]=e
								i=i+1
							end
						end
						if (table.getn(newOrder)==self.Menus) then
							self.db.profile.order = newOrder
							self.db.profile.button = newButton
							self:Reload()
						end
					end,
					message = L["MSG_MENU_ORDER_CHANGED"]
				},
				reset = {
					order = 500,
					type = "execute",
					name = L["Reset"],
					desc = L["reset_desc"],
					func = function()
						self:ResetDB("profile")
						self:Reload()
						self:Print(L["MSG_RESETED"])
					end
				}
			}
		}
	)

	--init for extended menu
	local _, class = UnitClass("player")
	self.SpellValid = {}
	self.SpellValid_Count = 0
	if SpellList[class] then
		for k,v in pairs(SpellList[class]) do
			self.SpellValid[bs[k]]=1
			self.SpellValid_Count = self.SpellValid_Count + 1
		end
	end
end

function PlayerMenu:OnEnable()
	self:InitSpell()
	self:RegisterEvent("SPELLS_CHANGED", "InitSpell")
	self:RegisterEvent("LEARNED_SPELL_IN_TAB", "InitSpell")

	self:SecureHook("UnitPopup_OnClick")
	self:SecureHook("UnitPopup_HideButtons")
	if (self.db.profile.leftButton) then
		self:SecureHook("SetItemRef")
	end

	UnitPopupButtons["ADD_FRIEND"] = { text = TEXT(ADD_FRIEND), dist = 0 }
	UnitPopupButtons["GUILD_INVITE"] = { text = L["TEXT_GUILD_INVITE"], dist = 0 }
	UnitPopupButtons["GET_NAME"] = { text = L["TEXT_GET_NAME"], dist = 0 }
	UnitPopupButtons["WHO"] = { text = TEXT(WHO), dist = 0 }
	UnitPopupButtons["EXTMENU"] = { text = L["Extended Menu"], dist = 0 }

	UnitPopupMenus["FRIEND"] = {}
	for k,v in pairs(self.db.profile.order) do
		if (self.db.profile.button[v]==1) then
			UnitPopupMenus["FRIEND"][k]=v
		end
	end

	tinsert(UnitPopupMenus["PLAYER"], getn(UnitPopupMenus["PLAYER"])-1, "ADD_FRIEND")
	tinsert(UnitPopupMenus["PLAYER"], getn(UnitPopupMenus["PLAYER"])-1, "GET_NAME")
	tinsert(UnitPopupMenus["PLAYER"], getn(UnitPopupMenus["PLAYER"])-1, "WHO")
	tinsert(UnitPopupMenus["PLAYER"], getn(UnitPopupMenus["PLAYER"])-1, "EXTMENU")

	tinsert(UnitPopupMenus["PARTY"], getn(UnitPopupMenus["PARTY"])-1, "ADD_FRIEND")
	tinsert(UnitPopupMenus["PARTY"], getn(UnitPopupMenus["PARTY"])-1, "GET_NAME")
	tinsert(UnitPopupMenus["PARTY"], getn(UnitPopupMenus["PARTY"])-1, "WHO")
	tinsert(UnitPopupMenus["PARTY"], getn(UnitPopupMenus["PARTY"])-1, "EXTMENU")

	tinsert(UnitPopupMenus["RAID_PLAYER"], getn(UnitPopupMenus["RAID_PLAYER"])-1, "ADD_FRIEND")
	tinsert(UnitPopupMenus["RAID_PLAYER"], getn(UnitPopupMenus["RAID_PLAYER"])-1, "GET_NAME")
	tinsert(UnitPopupMenus["RAID_PLAYER"], getn(UnitPopupMenus["RAID_PLAYER"])-1, "WHO")
	tinsert(UnitPopupMenus["RAID_PLAYER"], getn(UnitPopupMenus["RAID_PLAYER"])-1, "EXTMENU")
end

function PlayerMenu:OnDisable()
	UnitPopupButtons["ADD_FRIEND"] = nil
	UnitPopupButtons["GUILD_INVITE"] = nil
	UnitPopupButtons["GET_NAME"] = nil
	UnitPopupButtons["WHO"] = nil
	UnitPopupButtons["EXTMENU"] = nil

	UnitPopupMenus["FRIEND"]={}
	UnitPopupMenus["PLAYER"]={}
	UnitPopupMenus["PARTY"]={}
	UnitPopupMenus["RAID_PLAYER"]={}
	for k,v in pairs(self.UnitPopupMenus["FRIEND"]) do
		UnitPopupMenus["FRIEND"][k]=v
	end
	for k,v in pairs(self.UnitPopupMenus["PLAYER"]) do
		UnitPopupMenus["PLAYER"][k]=v
	end
	for k,v in pairs(self.UnitPopupMenus["PARTY"]) do
		UnitPopupMenus["PARTY"][k]=v
	end
	for k,v in pairs(self.UnitPopupMenus["RAID_PLAYER"]) do
		UnitPopupMenus["RAID_PLAYER"][k]=v
	end

	self:UnhookAll()
end

function PlayerMenu:UnitPopup_HideButtons()
	-- hide ext menu
	self:ExtMenu_AutoHide()

	local dropdownMenu = getglobal(UIDROPDOWNMENU_INIT_MENU)
	local diffFactionGroup
	if dropdownMenu.unit and UnitFactionGroup("player") ~= UnitFactionGroup(dropdownMenu.unit) then
		diffFactionGroup = true
	end
	local realm
	if dropdownMenu.unit then
		_,realm = UnitName(dropdownMenu.unit)
	end

	for index, value in ipairs(UnitPopupMenus[dropdownMenu.which]) do
		if value == "GET_NAME" then
			if dropdownMenu.name == UnitName("player") then
				UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 0
			else
				UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 1
			end
		elseif value == "WHO" or value == "ADD_FRIEND" then
			if dropdownMenu.name == UnitName("player") or realm or diffFactionGroup then
				UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 0
			else
				UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 1
			end
		elseif value == "GUILD_INVITE" then
			if not CanGuildInvite() or dropdownMenu.name == UnitName("player") or realm  or diffFactionGroup then
				UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 0
			else
				UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 1
			end
		elseif value == "EXTMENU" then
			UnitPopupShown[UIDROPDOWNMENU_MENU_LEVEL][index] = 1
		end
	end
end

function PlayerMenu:UnitPopup_OnClick()
	local dropdownFrame = getglobal(UIDROPDOWNMENU_INIT_MENU)
	local button = this.value
	local name = dropdownFrame.name

	if (button == "ADD_FRIEND") then
		AddFriend(name)
	elseif (button == "GUILD_INVITE") then
		GuildInvite(name)
	elseif (button == "GET_NAME") then
		ChatFrameEditBox:Show()
		local realm
		if dropdownFrame.unit then
			_,realm = UnitName(dropdownFrame.unit)
		end
		if realm then
			ChatFrameEditBox:Insert(name .. " - " .. realm)
		else
			ChatFrameEditBox:Insert(name)
		end
	elseif (button == "WHO") then
		SendWho(name)
	elseif (button == "EXTMENU") then
		self:ExtMenu_InitButton(name)
		self:ExtMenu_Show()
	end
	PlaySound("UChatScrollButton")
end

function PlayerMenu:SetItemRef(link, text, button)
	if ( strsub(link, 1, 6) == "player" ) then
		local namelink = strsub(link, 8);
		local name, lineid = strsplit(":", namelink);
		if ( name and (strlen(name) > 0) ) then
			name = gsub(name, "([^%s]*)%s+([^%s]*)%s+([^%s]*)", "%3");
			name = gsub(name, "([^%s]*)%s+([^%s]*)", "%2");
			if ( IsShiftKeyDown() ) then
				local staticPopup;
				staticPopup = StaticPopup_Visible("ADD_IGNORE");
				if ( staticPopup ) then
					-- If add ignore dialog is up then enter the name into the editbox
					getglobal(staticPopup.."EditBox"):SetText(name);
					return;
				end
				staticPopup = StaticPopup_Visible("ADD_FRIEND");
				if ( staticPopup ) then
					-- If add ignore dialog is up then enter the name into the editbox
					getglobal(staticPopup.."EditBox"):SetText(name);
					return;
				end
				staticPopup = StaticPopup_Visible("ADD_GUILDMEMBER");
				if ( staticPopup ) then
					-- If add ignore dialog is up then enter the name into the editbox
					getglobal(staticPopup.."EditBox"):SetText(name);
					return;
				end
				staticPopup = StaticPopup_Visible("ADD_RAIDMEMBER");
				if ( staticPopup ) then
					-- If add ignore dialog is up then enter the name into the editbox
					getglobal(staticPopup.."EditBox"):SetText(name);
					return;
				end
				if ( ChatFrameEditBox:IsVisible() ) then
					ChatFrameEditBox:Insert(name);
				else
					SendWho("n-"..name);
				end

			else
				ChatFrameEditBox:Hide()
				FriendsFrame_ShowDropdown(name, 1, lineid);
			end
		end
		return;
	end
end

function PlayerMenu:Reload()
	self:OnDisable()
	self:OnEnable()
end

function PlayerMenu:InitSpell()
	self.SpellCache = {}
	self.SpellCache_Count = 0

	if self.SpellValid_Count == 0 then
		return
	end

	local numTotalSpells = 0
	for i=1, MAX_SKILLLINE_TABS do
		local name, _, _, numSpells = GetSpellTabInfo(i)
		if name then
			numTotalSpells = numTotalSpells + numSpells
		end
	end

	local SpellExists = {}
	for i=1, numTotalSpells do
		local spellName = GetSpellName(i, BOOKTYPE_SPELL)
		if spellName and self.SpellValid[spellName] and not SpellExists[spellName] then
			table.insert(self.SpellCache, spellName)
			SpellExists[spellName] = 1
		end
	end
	self.SpellCache_Count = getn(self.SpellCache)
end
function PlayerMenu:ExtMenu_ButtonOnClick()
	PMEM_Frame:Hide()
end
function PlayerMenu:ExtMenu_InitButton(name)
	if not name then
		name = UnitName("player")
	end
	PMEM_Frame_Button1:SetText(L["Set Target"])
	PMEM_Frame_Button1:SetAttribute("type1", "macro")
	PMEM_Frame_Button1:SetAttribute("macrotext", "/target "..name)
	PMEM_Frame_Button1:Show()
	PMEM_Frame_Button2:SetText(L["Set Focus"])
	PMEM_Frame_Button2:SetAttribute("type1", "macro")
	PMEM_Frame_Button2:SetAttribute("macrotext", "/focus "..name)
	PMEM_Frame_Button2:Show()

	local spell_id = 1
	for i=3, self.SpellCache_Count+2,1 do
		local button = getglobal("PMEM_Frame_Button"..i)
		button:SetText(self.SpellCache[spell_id])
		button:SetAttribute("type1", "macro")
		if self.db.profile.useFocus then
			button:SetAttribute("macrotext", "/focus "..name.."\n/cast [target=focus] "..self.SpellCache[spell_id].."\n/stopcasting\n/clearfocus")
		else
			local lastTarget = UnitName("target")
			if lastTarget then
				button:SetAttribute("macrotext", "/target "..name.."\n/cast "..self.SpellCache[spell_id].."\n/stopcasting\n/target "..lastTarget)
			else
				button:SetAttribute("macrotext", "/target "..name.."\n/cast "..self.SpellCache[spell_id].."\n/stopcasting\n")
			end
		end
		button:Show()
		spell_id = spell_id + 1
	end

	for i=self.SpellCache_Count+3,20,1 do
		local button = getglobal("PMEM_Frame_Button"..i)
		button:Hide()
	end

	self.ExtMenuWidth = 0
	for i=1,20,1 do
		local button = getglobal("PMEM_Frame_Button"..i)
		local width = button:GetTextWidth()
		if width > self.ExtMenuWidth then
			self.ExtMenuWidth = width
		end
	end
	self.ExtMenuWidth = self.ExtMenuWidth + 2*12
	self.ExtMenuHeight = (self.SpellCache_Count + 4) * 12
end
function PlayerMenu:ExtMenu_AutoHide()
	if PlayerMenu:IsEventScheduled("PlayerMenu_ExtMenu_AutoHide") then
		PlayerMenu:CancelScheduledEvent("PlayerMenu_ExtMenu_AutoHide")
	end
	PMEM_Frame:Hide()
end
function PlayerMenu:ExtMenu_Show()
	self:ScheduleRepeatingEvent("PlayerMenu_ExtMenu_AutoHide", self.ExtMenu_AutoHide, self.db.profile.extAutoHide)
	PMEM_Frame:SetHeight(self.ExtMenuHeight)
	PMEM_Frame:SetWidth(self.ExtMenuWidth)
	PMEM_Frame:ClearAllPoints()
	PMEM_Frame:SetPoint("TOPLEFT", "DropDownList1", "TOPLEFT")
	PMEM_Frame:Show()
end
function PlayerMenu:ExtMenu_OnEnter()
	if self:IsEventScheduled("PlayerMenu_ExtMenu_AutoHide") then
		self:CancelScheduledEvent("PlayerMenu_ExtMenu_AutoHide")
	end
end
function PlayerMenu:ExtMenu_OnLeave()
	if not self:IsEventScheduled("PlayerMenu_ExtMenu_AutoHide") then
		self:ScheduleRepeatingEvent("PlayerMenu_ExtMenu_AutoHide", self.ExtMenu_AutoHide, self.db.profile.extAutoHide)
	end
end